@@ -15,6 +15,7 @@
 // You should have received a copy of the GNU General Public License
 // along with spat. If not, see <http://www.gnu.org/licenses/>.
 
+
 #include <numeric>
 #include "geos_spat.h"
 #include "distance.h"
@@ -289,7 +290,7 @@ std::vector<std::vector<unsigned>> SpatVector::index_sparse(SpatVector v) {
 
 
 
-SpatVector SpatVector::crop(SpatExtent e) {
+SpatVector SpatVector::crop(SpatExtent e, bool wrap) {
 
 	SpatVector out;
 
@@ -297,10 +298,35 @@ SpatVector SpatVector::crop(SpatExtent e) {
 	out.setError("GEOS 3.5 required for crop");
 	return out;
 #else
+	if (is_lonlat() & wrap) {
+		SpatVector first = crop(e, false);
+		if ((extent.xmin > -180.001) && (extent.xmax < 180.001)) {
+			if ((e.xmin < -180) && (e.xmax < 180)) {
+				double xmn = e.xmin + 360;
+				if (xmn > -180) {
+					if (xmn < e.xmax) xmn = e.xmax;
+					SpatExtent e2 = e;
+					e2.xmax = 180;
+					e2.xmin = xmn;
+					SpatVector second = crop(e2, false);
+					first = first.append(second, true);
+				}
+			} else if ((e.xmax > 180) && (e.xmin > -180)) {
+				double xmx = -360 + e.xmax;
+				if (xmx < 180) {
+					if (xmx > e.xmin) xmx = e.xmin;
+					SpatExtent e2 = e;
+					e2.xmin = -180;
+					e2.xmax = xmx;
+					SpatVector second = crop(e2, false);
+					first = first.append(second, true);
+				}
+			} 
+		}
+		return(first);						
+	}
 
-	out.srs = srs;
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
-
 	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
 	std::vector<GeomPtr> p;
 	p.reserve(g.size());
@@ -320,13 +346,14 @@ SpatVector SpatVector::crop(SpatExtent e) {
 			GEOSGeom_destroy_r(hGEOSCtxt, r);
 		}
 	}
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt, id);
 		out = coll.get(0);
 		out.df = df.subset_rows(out.df.iv[0]);
-		out.srs = srs;
 	}
 	geos_finish(hGEOSCtxt);
+	
+	out.srs = srs;
 	return out;
 #endif
 }
@@ -355,7 +382,7 @@ SpatVector SpatVector::make_nodes() {
 			GEOSGeom_destroy_r(hGEOSCtxt, r);
 		}
 	}
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt);
 		out = coll.get(0);
 		out.df = df;
@@ -389,7 +416,7 @@ SpatVector SpatVector::boundary() {
 			GEOSGeom_destroy_r(hGEOSCtxt, r);
 		}
 	}
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt);
 		out = coll.get(0);
 		out.df = df;
@@ -451,7 +478,7 @@ SpatVector SpatVector::line_merge() {
 			GEOSGeom_destroy_r(hGEOSCtxt, r);
 		}
 	}
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt);
 		out = coll.get(0);
 		out.df = df;
@@ -488,7 +515,7 @@ SpatVector SpatVector::simplify(double tolerance, bool preserveTopology) {
 			GEOSGeom_destroy_r(hGEOSCtxt, r);
 		}
 	}
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt);
 		out = coll.get(0);
 		out.df = df;
@@ -564,7 +591,7 @@ SpatVector SpatVector::shared_paths(bool index) {
 	}
 
 	SpatVector out;
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt, std::vector<long>(), false, false);
 		out = coll.get(0);
 		out = out.line_merge();
@@ -644,7 +671,7 @@ SpatVector SpatVector::shared_paths(SpatVector x, bool index) {
 	}
 
 	SpatVector out;
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt, std::vector<long>(), false, false);
 		out = coll.get(0);
 		out = out.line_merge();
@@ -698,7 +725,7 @@ SpatVector SpatVector::polygonize() {
 			GEOSGeom_destroy_r(hGEOSCtxt, r);
 		}
 	}
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt);
 		out = coll.get(0);
 		out.srs = srs;
@@ -838,7 +865,7 @@ SpatVector SpatVector::crop(SpatVector v) {
 
 //	SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt);
 
-	if (result.size() > 0) {
+	if (!result.empty()) {
 //		SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt);
 		SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt, ids);
 		out = coll.get(0);
@@ -855,7 +882,7 @@ SpatVector SpatVector::hull(std::string htype, std::string by) {
 
 	SpatVector out;
 
-	if (by != "") {
+	if (!by.empty()) {
 		SpatVector tmp = aggregate(by, false);
 		if (tmp.hasError()) {
 			return tmp;
@@ -866,7 +893,7 @@ SpatVector SpatVector::hull(std::string htype, std::string by) {
 			if (x.hasError()) {
 				return x;
 			}
-			if ((x.geoms.size() > 0) && (x.geoms[0].gtype == polygons)) {
+			if (!x.geoms.empty() && (x.geoms[0].gtype == polygons)) {
 				out.addGeom(x.geoms[0]);
 			} else {
 				SpatGeom g(polygons);
@@ -887,9 +914,9 @@ SpatVector SpatVector::hull(std::string htype, std::string by) {
 		#endif
 		if (is_lonlat()) {
 			if ((extent.ymin > -85) && (extent.ymax < 85)) {
-				SpatVector tmp = project("+proj=merc");
+				SpatVector tmp = project("+proj=merc", false);
 				tmp = tmp.hull(htype, "");
-				tmp = tmp.project(srs.wkt);
+				tmp = tmp.project(srs.wkt, false);
 				return tmp;
 			}
 		}
@@ -935,16 +962,16 @@ SpatVector SpatVector::voronoi(SpatVector bnd, double tolerance, int onlyEdges)
 	SpatVector a = aggregate(false);
 	std::vector<GeomPtr> g = geos_geoms(&a, hGEOSCtxt);
 	GEOSGeometry* v;
-	if (bnd.size() > 0) {
+	if (bnd.empty()) {
+		v = GEOSVoronoiDiagram_r(hGEOSCtxt, g[0].get(), NULL, tolerance, onlyEdges);
+	} else {
 		if (bnd.type() != "polygons") {
-			out.setError("boundary must be polygon");
+			out.setError("boundary must have a polygon geometry");
 			geos_finish(hGEOSCtxt);
 			return out;
 		}
 		std::vector<GeomPtr> ge = geos_geoms(&bnd, hGEOSCtxt);
 		v = GEOSVoronoiDiagram_r(hGEOSCtxt, g[0].get(), ge[0].get(), tolerance, onlyEdges);
-	} else {
-		v = GEOSVoronoiDiagram_r(hGEOSCtxt, g[0].get(), NULL, tolerance, onlyEdges);
 	}
 	if (v == NULL) {
 		out.setError("GEOS exception");
@@ -959,7 +986,7 @@ SpatVector SpatVector::voronoi(SpatVector bnd, double tolerance, int onlyEdges)
 	out.srs = srs;
 	if (!out.hasError()) {
 		out = out.disaggregate(false);
-		if (bnd.size() > 0) {
+		if (!bnd.empty()) {
 			SpatDataFrame empty;
 			bnd.df = empty;
 			out = out.intersect(bnd, true);
@@ -1042,7 +1069,7 @@ SpatVector lonlat_buf(SpatVector x, double dist, unsigned quadsegs, bool ispol,
 		std::vector<double> dd = destpoint_lonlat(0, halfy, 0, dist);
 		dist = dd[1] - halfy;
 		if (ishole) dist = -dist;
-		x = x.buffer({dist}, quadsegs);
+		x = x.buffer({dist}, quadsegs, "", "", NAN, false);	
 		x.srs = insrs;
 		return x;
 	}
@@ -1094,33 +1121,33 @@ SpatVector lonlat_buf(SpatVector x, double dist, unsigned quadsegs, bool ispol,
 }
 
 
-
-SpatVector SpatVector::buffer(std::vector<double> dist, unsigned quadsegs) {
+//, std::vector<std::string> pars
+SpatVector SpatVector::buffer(std::vector<double> d, unsigned quadsegs, std::string capstyle, std::string joinstyle, double mitrelimit, bool singlesided) {
 
 	quadsegs = std::min(quadsegs, (unsigned) 180);
 	SpatVector out;
-	out.srs = srs;
+//	out.srs = srs;
 	if (srs.is_empty()) {
 		out.addWarning("unknown CRS. Results may be wrong");
 	} 
 	bool islonlat = is_lonlat();
 	
-	if (dist.size() == 1 && dist[0] == 0) {
+	if (d.size() == 1 && d[0] == 0) {
 		islonlat = false; //faster
 	}
 	std::string vt = type();
 	if (vt == "points" || vt == "lines") {
-		for (size_t i=0; i<dist.size(); i++) {
-			if (dist[i] <= 0) {
-				dist[i] = -dist[i];
+		for (size_t i=0; i<d.size(); i++) {
+			if (d[i] <= 0) {
+				d[i] = -d[i];
 			}
 		}
 	}
-	recycle(dist, size());
+	recycle(d, size());
 
 	if (islonlat) {
 		if (vt == "points") {
-			return point_buffer(dist, quadsegs, false);
+			return point_buffer(d, quadsegs, false);
 		} else {
 			SpatVector p;
 			bool ispol = vt == "polygons";
@@ -1129,34 +1156,58 @@ SpatVector SpatVector::buffer(std::vector<double> dist, unsigned quadsegs) {
 				if (ispol) {
 					SpatVector h = p.get_holes();
 					p = p.remove_holes();
-					p = lonlat_buf(p, dist[i], quadsegs, true, false);
-					if (h.size() > 0) {
-						h = lonlat_buf(h, dist[i], quadsegs, true, true);
-						if (h.size() > 0) {
+					p = lonlat_buf(p, d[i], quadsegs, true, false);
+					if (!h.empty()) {
+						h = lonlat_buf(h, d[i], quadsegs, true, true);
+						if (!h.empty()) {
 							for (size_t j=0; j<h.geoms[0].parts.size(); j++) {
 								p.geoms[0].parts[0].addHole(h.geoms[0].parts[j].x, h.geoms[0].parts[j].y);
 							}
 						}
 					}
 				} else {
-					p = lonlat_buf(p, dist[i], quadsegs, false, false);
+					p = lonlat_buf(p, d[i], quadsegs, false, false);
 				}
 				out = out.append(p, true);
 			}
 			out.df = df;
+			out.srs = srs;
 			return out;
 		}
 	}
 
 
-
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 //	SpatVector f = remove_holes();
 
+	GEOSBufferParams* bufparms = GEOSBufferParams_create_r(hGEOSCtxt);	
+	GEOSBufferParams_setQuadrantSegments_r(hGEOSCtxt, bufparms, quadsegs);
+	if (capstyle == "flat") {
+		GEOSBufferParams_setEndCapStyle_r(hGEOSCtxt, bufparms, GEOSBUF_CAP_FLAT);
+	} else if (capstyle == "square") { 
+		GEOSBufferParams_setEndCapStyle_r(hGEOSCtxt, bufparms, GEOSBUF_CAP_SQUARE);
+	} else {
+		GEOSBufferParams_setEndCapStyle_r(hGEOSCtxt, bufparms, GEOSBUF_CAP_ROUND);
+	}
+	if (joinstyle == "mitre") {
+		GEOSBufferParams_setJoinStyle_r(hGEOSCtxt, bufparms, GEOSBUF_JOIN_MITRE);
+	} else if (joinstyle == "bevel") { 
+		GEOSBufferParams_setJoinStyle_r(hGEOSCtxt, bufparms, GEOSBUF_JOIN_BEVEL);
+	} else {
+		GEOSBufferParams_setJoinStyle_r(hGEOSCtxt, bufparms, GEOSBUF_JOIN_ROUND);
+	}
+	if (!std::isnan(mitrelimit)) { 
+		GEOSBufferParams_setMitreLimit_r(hGEOSCtxt, bufparms, mitrelimit);
+	}
+	if (singlesided) { 
+		GEOSBufferParams_setSingleSided_r(hGEOSCtxt, bufparms, 1);
+	} 
+	
 	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
 	std::vector<GeomPtr> b(size());
 	for (size_t i = 0; i < g.size(); i++) {
-		GEOSGeometry* pt = GEOSBuffer_r(hGEOSCtxt, g[i].get(), dist[i], quadsegs);
+		GEOSGeometry* pt = GEOSBufferWithParams_r(hGEOSCtxt, g[i].get(), bufparms, d[i]);		
+//		GEOSGeometry* pt = GEOSBuffer_r(hGEOSCtxt, g[i].get(), d[i], quadsegs);
 		if (pt == NULL) {
 			out.setError("GEOS exception");
 			geos_finish(hGEOSCtxt);
@@ -1166,6 +1217,9 @@ SpatVector SpatVector::buffer(std::vector<double> dist, unsigned quadsegs) {
 	}
 	SpatVectorCollection coll = coll_from_geos(b, hGEOSCtxt);
 
+
+	GEOSBufferParams_destroy_r(hGEOSCtxt, bufparms);	
+
 //	out = spat_from_geom(hGEOSCtxt, g, "points");
 	geos_finish(hGEOSCtxt);
 	out = coll.get(0);
@@ -1241,7 +1295,7 @@ SpatVector SpatVector::intersect(SpatVector v, bool values) {
 
 
 	//SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt);
-		if (result.size() > 0) {
+		if (!result.empty()) {
 			SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt, ids, true, false);
 			out = coll.get(0);
 			out.srs = srs;
@@ -2158,7 +2212,7 @@ SpatVector SpatVector::mask(SpatVector x, bool inverse) {
 typedef int (* dist_fn)(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *, double *);
 
 bool get_dist_fun(dist_fn &f, std::string s) {
-	if ((s == "Euclidean") || (s == ""))
+	if ((s == "Euclidean") || s.empty())
 		f = GEOSDistance_r;
 	else if (s == "Hausdorff")
 		f = GEOSHausdorffDistance_r;
@@ -2203,7 +2257,7 @@ std::vector<double> SpatVector::geos_distance(SpatVector v, bool parallel, std::
 				std::swap(nx, ny);
 				nyone = true;
 			} else {
-				setError("vectors have different lengths");
+				setError("SpatVectors have different lengths");
 				return out;
 			}
 		}
@@ -2288,7 +2342,8 @@ std::vector<double> SpatVector::geos_distance(bool sequential, std::string fun)
 SpatVector SpatVector::unite(SpatVector v) {
 
 	SpatVector out;
-	if (type() != v.type()) {
+	std::string gtp = type();
+	if (gtp != v.type()) {
 		out.setError("cannot unite different geom types");
 		return out;
 	}
@@ -2300,14 +2355,16 @@ SpatVector SpatVector::unite(SpatVector v) {
 	if (out.nrow() == 0) {
 		return append(v, true);
 	}
-
+	if (gtp != out.type()) {
+		SpatVector empty;
+		out = empty;
+	}
 	SpatVector sdif = symdif(v);
 	if (sdif.hasError()) {
 		return sdif;
 	}
-
-	if (sdif.type() == type()) {
-		return out.append(sdif, true);
+	if ((sdif.nrow() > 0) && (gtp == sdif.type())) {	
+		return sdif.append(out, true);
 	}
 	return out;
 }
@@ -2480,7 +2537,7 @@ SpatVector SpatVector::erase_agg(SpatVector v) {
 			rids.push_back(i);
 		}
 	}
-	if (result.size() > 0) {
+	if (!result.empty()) {
 		SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt);
 		out = coll.get(0);
 		out.srs = srs;
@@ -2539,7 +2596,10 @@ SpatVector SpatVector::erase(SpatVector v) {
 		if (good) rids.push_back(i);
 	}
 
-	if (rids.size() > 0) {
+	if (rids.empty()) {
+		std::vector<int> none(1, -1);
+		out = subset_rows(none);
+	} else {
 		SpatVectorCollection coll = coll_from_geos(x, hGEOSCtxt);
 		out = coll.get(0);
 		out.srs = srs;
@@ -2547,9 +2607,6 @@ SpatVector SpatVector::erase(SpatVector v) {
 		if (rids.size() != out.nrow()) {
 			out = out.subset_rows(rids);
 		}
-	} else {
-		std::vector<int> none(1, -1);
-		out = subset_rows(none);
 	}
 	geos_finish(hGEOSCtxt);
 	if (!srs.is_same(v.srs, true)) {
@@ -2730,7 +2787,7 @@ SpatVector SpatVector::gaps() {
 SpatVector SpatVector::nearest_point(SpatVector v, bool parallel) {
 	SpatVector out;
 
-	if ((size() == 0) || (v.size()==0)) {
+	if ((size() == 0) || v.empty()) {
 		out.setError("empty SpatVecor(s)");
 		return out;
 	}
@@ -2799,8 +2856,8 @@ SpatVector SpatVector::nearest_point() {
 		return out;
 	}
 	if ((size() == 1)) {
-		out.addWarning("single geometry");
-		//return *this;
+		out.setError("single geometry");
+		return out;
 	}
 	size_t n = size();
 	out.srs = srs;
