@@ -25,7 +25,7 @@
 #endif
 
 
-enum SpatGeomType { points, lines, polygons, unknown, null };
+enum SpatGeomType { points, lines, polygons, null};
 
 
 class SpatHole {
@@ -38,7 +38,8 @@ class SpatHole {
 		SpatHole();
 		SpatHole(std::vector<double> X, std::vector<double> Y);
 		//methods
-		size_t size() { return x.size(); }	
+		size_t size() { return x.size(); }
+		bool empty() { return x.empty(); }
 };
 
 class SpatPart {
@@ -55,11 +56,12 @@ class SpatPart {
 
 		//methods
 		size_t size() { return x.size(); }
+		bool empty() { return x.empty(); }
 		//holes, polygons only
 		bool addHole(std::vector<double> X, std::vector<double> Y);
 		bool addHole(SpatHole h);
 		SpatHole getHole(unsigned i) { return( holes[i] ) ; }
-		bool hasHoles() { return holes.size() > 0;}
+		bool hasHoles() { return !holes.empty();}
 		unsigned nHoles() { return holes.size();}
 		size_t ncoords();
 };
@@ -73,7 +75,7 @@ class SpatGeom {
 		SpatGeom(SpatPart p, SpatGeomType type);
 		virtual ~SpatGeom(){}
 
-		SpatGeomType gtype = unknown;
+		SpatGeomType gtype = null;
 		std::vector<SpatPart> parts;
 		SpatExtent extent;
 
@@ -89,6 +91,7 @@ class SpatGeom {
 		//double length_plane();
 		//double length_lonlat(double a, double f);
 		unsigned size() { return parts.size(); };
+		bool empty() { return parts.empty(); };
 		void remove_duplicate_nodes(int digits);
 		size_t ncoords();
 		std::vector<std::vector<double>> coordinates();
@@ -171,7 +174,7 @@ class SpatVector {
 		size_t ncoords();
 		std::vector<std::vector<double>> coordinates();
 
-		SpatVector project(std::string crs);
+		SpatVector project(std::string crs, bool partial);
 		std::vector<double> project_xy(std::vector<double> x, std::vector<double> y, std::string fromCRS, std::string toCRS);
 
 		SpatVector subset_cols(int i);
@@ -196,11 +199,12 @@ class SpatVector {
 
 
 		std::vector<double> distance(bool sequential, std::string unit);
-		std::vector<double> linedistLonLat(SpatVector pts);
+		std::vector<double> linedistLonLat(SpatVector x);
 
 		std::vector<std::vector<size_t>> knearest(size_t k);
 
 		size_t size();
+		bool empty();
 		SpatVector as_lines();
 		SpatVector as_points(bool multi, bool skiplast=false);
 		SpatVector remove_holes();
@@ -215,7 +219,7 @@ class SpatVector {
 #ifdef useGDAL
 		GDALDataset* write_ogr(std::string filename, std::string lyrname, std::string driver, bool append, bool overwrite, std::vector<std::string> options);
 		GDALDataset* GDAL_ds();
-		bool read_ogr(GDALDataset *poDS, std::string layer, std::string query, std::vector<double> extent, SpatVector filter, bool as_proxy, std::string what);
+		bool read_ogr(GDALDataset *&poDS, std::string layer, std::string query, std::vector<double> extent, SpatVector filter, bool as_proxy, std::string what);
 		SpatVector fromDS(GDALDataset *poDS);
 		bool ogr_geoms(std::vector<OGRGeometryH> &ogrgeoms, std::string &message);		
 		bool delete_layers(std::string filename, std::vector<std::string> layers, bool return_error);		
@@ -323,15 +327,16 @@ class SpatVector {
 		SpatVector aggregate(bool dissolve);
 		SpatVector aggregate(std::string field, bool dissolve);
 
-        SpatVector buffer(std::vector<double> d, unsigned quadsegs);
+		SpatVector buffer(std::vector<double> d, unsigned quadsegs, std::string capstyle, std::string joinstyle, double mitrelimit, bool singlesided);
+
 		SpatVector point_buffer(std::vector<double>	 d, unsigned quadsegs, bool no_multipolygons);
 
 		SpatVector centroid(bool check_lonlat);
 		SpatVector point_on_surface(bool check_lonlat);
 
-		SpatVector crop(SpatExtent e);
+		SpatVector crop(SpatExtent e, bool wrap);
 		SpatVector crop(SpatVector e);
-		SpatVector voronoi(SpatVector e, double tolerance, int onlyEdges);		
+		SpatVector voronoi(SpatVector bnd, double tolerance, int onlyEdges);		
 		SpatVector delaunay(double tolerance, int onlyEdges);		
 		SpatVector hull(std::string htype, std::string by="");
 		SpatVector intersect(SpatVector v, bool values);
@@ -340,7 +345,7 @@ class SpatVector {
 		SpatVector erase_agg(SpatVector v);
 		SpatVector erase(SpatVector v);
 		SpatVector erase(bool sequential);
-		SpatVector elongate(double length);
+		SpatVector elongate(double length, bool flat);
 		SpatVector mask(SpatVector x, bool inverse);
 		SpatVector gaps();		
 		SpatVector cover(SpatVector v, bool identity, bool expand);
@@ -381,6 +386,7 @@ class SpatVector {
 		SpatVector densify(double interval, bool adjust, bool ignorelonlat);
 		SpatVector round(int digits);
 		std::vector<unsigned> nullGeoms();
+				
 };
 
 
@@ -390,7 +396,16 @@ class SpatVectorCollection {
 
 	public:
 		virtual ~SpatVectorCollection(){}
+		SpatVectorCollection();
+		SpatVectorCollection(std::string filename, std::string layer, std::string query, std::vector<double> extent, SpatVector filter);
+		
+		
 		SpatVectorCollection deepCopy() { return *this; }
+		bool read(std::string fname, std::string layer, std::string query, std::vector<double> extent, SpatVector filter);
+		
+		bool read_ogr(GDALDataset *&poDS, std::string layer, std::string query, std::vector<double> extent, SpatVector filter);
+
+//		SpatVectorCollection create(std::string filename);
 
 		std::vector<SpatVector> v;
 		std::vector<std::string> names;
@@ -406,6 +421,7 @@ class SpatVectorCollection {
 		std::string getError() { return msg.getError();}
 
 		size_t size() { return v.size(); }
+		bool empty() { return v.empty(); }
 		void reserve(size_t n) { v.reserve(n); names.reserve(n); }
 		void resize(size_t n) { v.resize(n); names.resize(n); }
 		void push_back(SpatVector x) {
